var test = require('tape');
var browserify = require( 'browserify' );
var parcelify = require('../');
var os = require('os');
var path = require('path');
var fs = require('fs');
var mkdirp = require('mkdirp');
var tmpdir = (os.tmpdir || os.tmpDir)();

test( 'page1', function( t ) {
	t.plan( 2 );
	
	var mainPath = __dirname + '/page1/main.js';
	
	var dstDir = path.resolve( tmpdir, 'parcelify-test-' + Math.random() );

	var options = {
		bundles : {
			// script : path.join( dstDir, 'bundle.js' ),
			style : path.join( dstDir, 'bundle.css' )
		}
	};

	mkdirp.sync( dstDir );

    var b = browserify( mainPath );
	var p = parcelify( b, options );

	b.bundle();

	p.on( 'done', function() {
		t.deepEqual(
			fs.readdirSync( dstDir ).sort(),
			[ 'bundle.css' ]
		);

		t.deepEqual( fs.readFileSync( options.bundles.style, 'utf8' ), 'h1 {\n\tfont-size: 18px;\n}body {\n	color: red;\n}\n' );
	} );
} );

test( 'page2', function( t ) {
	t.plan( 2 );
	
	var mainPath = __dirname + '/page2/index.js';
	
	var dstDir = path.resolve( tmpdir, 'parcelify-test-' + Math.random() );
	var options = {
		bundles : {
			style : path.join( dstDir, 'bundle.css' )
		}
	};

	mkdirp.sync( dstDir );

    var b = browserify( mainPath );
	var p = parcelify( b, options );

	b.bundle();

	p.on( 'done', function() {
		t.deepEqual(
			fs.readdirSync( dstDir ).sort(),
			[ 'bundle.css' ]
		);

		t.deepEqual( fs.readFileSync( options.bundles.style, 'utf8' ), 'h1 {\n\tfont-size: 18px;\n}h2 {\n\tfont-weight: bold;\n}' );
	} );
} );

test( 'page3', function( t ) {
	t.plan( 2 );
	
	var mainPath = __dirname + '/page3/index.js';
	
	var dstDir = path.resolve( tmpdir, 'parcelify-test-' + Math.random() );
	var options = {
		bundles : {
			style : path.join( dstDir, 'bundle.css' )
		}
	};

	mkdirp.sync( dstDir );

    var b = browserify( mainPath );
	var p = parcelify( b, options );

	b.bundle();

	p.on( 'done', function() {
		t.deepEqual(
			fs.readdirSync( dstDir ).sort(),
			[ 'bundle.css' ]
		);

		// makes sure when multiple files are listed in a package.json style property, they are concatenated in the right order
		// (in this case in my-other-module, style is [ "myOtherModuleRed.css", "myOtherModuleBlue.css" ] )
		t.deepEqual( fs.readFileSync( options.bundles.style, 'utf8' ), 'h3 {\n\tcolor: red;\n}h2 {\n\tcolor: blue;\n}' );
	} );
} );

test( 'page4', function( t ) {
	t.plan( 3 );
	
	var mainPath = __dirname + '/page4/main.js';
	
	var dstDir = path.resolve( tmpdir, 'parcelify-test-' + Math.random() );
	var options = {
		bundles : {
			script : path.join( dstDir, 'bundle.js' ),
			style : path.join( dstDir, 'bundle.css' ),
			template : path.join( dstDir, 'bundle.tmpl' )
		}
	};

	mkdirp.sync( dstDir );

	var b = browserify( mainPath );
	var p = parcelify( b, options );

	b.bundle();

	p.on( 'done', function() {
		t.deepEqual(
			fs.readdirSync( dstDir ).sort(),
			[ 'bundle.css', 'bundle.tmpl' ]
		);

		t.deepEqual( fs.readFileSync( options.bundles.style, 'utf8' ), 'h1 {\n\tfont-size: 18px;\n}body h3 {\n  color: red; }\n' );
		t.deepEqual( fs.readFileSync( options.bundles.template, 'utf8' ), '<script type="template" id="my-module">\n<p>I am a template in myModule.</p>\n</script>\n<script type="template" id="my-template">\n<p>Hello There! I am a template.</p>\n</script>\n' );
	} );
} );

// test a parcel with no package.json and no options
test( 'page5', function( t ) {
	t.plan( 1 );
	
	var mainPath = __dirname + '/page5/main.js';
	
	var dstDir = path.resolve( tmpdir, 'parcelify-test-' + Math.random() );

	mkdirp.sync( dstDir );

	var b = browserify( mainPath );
	var p = parcelify( b );

	b.bundle();

	p.on( 'done', function() {
		t.deepEqual(
			fs.readdirSync( dstDir ).sort(),
			[]
		);
	} );

} );


// default transforms
test( 'page6', function( t ) {
	t.plan( 2 );
	
	var mainPath = __dirname + '/page6/main.js';
	
	var dstDir = path.resolve( tmpdir, 'parcelify-test-' + Math.random() );

	var options = {
		bundles : {
			style : path.join( dstDir, 'bundle.css' )
		},

		appTransforms : [ require( 'sass-css-stream' ) ],
		appTransformDirs : [ './test/page6/' ]
	};

	mkdirp.sync( dstDir );

	var b = browserify( mainPath );
	var p = parcelify( b, options );

	b.bundle();

	p.on( 'done', function() {
		t.deepEqual(
			fs.readdirSync( dstDir ).sort(),
			[ 'bundle.css' ]
		);

		t.deepEqual( fs.readFileSync( options.bundles.style, 'utf8' ), 'div.beep div.boop {\n  color: green; }\n' );
	} );
} );


// ordering
test( 'page7', function( t ) {
	t.plan( 2 );
	
	var mainPath = __dirname + '/page7/main.js';
	
	var dstDir = path.resolve( tmpdir, 'parcelify-test-' + Math.random() );

	var options = {
		bundles : {
			style : path.join( dstDir, 'bundle.css' )
		}
	};

	mkdirp.sync( dstDir );

	var b = browserify( mainPath );
	var p = parcelify( b, options );

	b.bundle();

	p.on( 'done', function() {
		t.deepEqual(
			fs.readdirSync( dstDir ).sort(),
			[ 'bundle.css' ]
		);

		t.deepEqual( fs.readFileSync( options.bundles.style, 'utf8' ), 'p {\n\tcolor: red;\n}p.header {\n\tcolor: blue;\n}p.footer {\n\tcolor: green;\n}div.beep {\n\tcolor : green;\n}' );
	} );
} );